#include "stdafx.h"
#define MAX_CHAR 100
#define ERROR_ARGUMENT_NUMBER 1
#define ERROR_ARGUMENT_TYPE 2
#define ERROR_CANNOT_OPEN_FILE 3
#define ERROR_CANNOT_CLOSE_FILE 4

typedef struct{
  TCHAR Kanji[MAX_CHAR];
  TCHAR Hiragana[MAX_CHAR];
  int IsMale;
} name;

#ifdef _MBCS
  const TCHAR PadCharacter = _T('*');
  const int NameColumnWidth = 12;
#endif
#ifdef _UNICODE
  const TCHAR PadCharacter = _T('');
  const int NameColumnWidth = 6;
#endif

void showVersion();
int makeName(int howMucnData);
int getRandomNumber(int max);
TCHAR *fixLength(TCHAR *s);

int _tmain(int argc, _TCHAR* argv[]) {
  int result = 0;
  _tsetlocale(LC_ALL, _T(""));

  if ((argc < 2) || (argc > 3)) {
    result = ERROR_ARGUMENT_NUMBER;
  } else {
    if (argv[1][0] == _T('-')) {
      switch (argv[1][1]) {
        case _T('v'):          
          showVersion();
          break;
        default:
          result = ERROR_ARGUMENT_TYPE;
      }
    } else {
      result = makeName(_tstoi(argv[1]));
    }
  }

  switch (result) {
    case ERROR_ARGUMENT_NUMBER:
      _ftprintf(stderr, _T("R}hČႢ܂\n"));
      break;
    case ERROR_ARGUMENT_TYPE:
      _ftprintf(stderr, _T("R}hCsK؂ł\n"));
      break;
    case ERROR_CANNOT_OPEN_FILE:
      _ftprintf(stderr, _T("t@CJ܂ł"));
      break;
    case ERROR_CANNOT_CLOSE_FILE:
      _ftprintf(stderr, _T("t@C܂ł"));
      break;
  }
  return result;
}

void showVersion() {
  _TCHAR mode = _T('s');

  #ifdef _MBCS
    mode = _T('m');
  #endif
  #ifdef _UNICODE
    mode = _T('u');
  #endif
  _tprintf(_T("%s%c %s\n"), _T("Test Data Generator Version 0.01"),
    mode, _T("(c)2009 Hideo Harada\n"));
  return;
}

int makeName(int howMuchData) {
  int result = 0;
  FILE *stream;
  errno_t err;
  _TCHAR inputLine[MAX_CHAR];
  int firstNameCount = 0, lastNameCount = 0;
  name *firstName = NULL, *firstNameInitialValue = NULL;
  name *lastName = NULL, *lastNameInitialValue = NULL;

  err = _tfopen_s(&stream, _T("lastname.txt"), _T("r"));
  if (err != 0) {
    result = ERROR_CANNOT_OPEN_FILE;
    goto error;
  }

  while (NULL != _fgetts(inputLine, MAX_CHAR, stream)) lastNameCount++;
  rewind(stream);
  
  lastNameInitialValue = (name *) calloc(lastNameCount, sizeof(name));
  lastName = lastNameInitialValue;
  while (NULL != _fgetts(inputLine, MAX_CHAR, stream)) {
    int column = 0;
    int index = 0;
    for (unsigned int i = 0; i < _tcslen(inputLine); i++) {
      if ((inputLine[i] == ',') || (inputLine[i] == _T('\n'))) {
        switch (column) {
          case 0:
            lastName->Kanji[index] = _T('\0');
            index = 0;
            break;
          case 1:
            lastName->Hiragana[index] = _T('\0');
            break;
        }
        column++;
      } else {
        switch (column) {
          case 0:
            lastName->Kanji[index] = inputLine[i];
            index++;
            break;
          case 1:
            lastName->Hiragana[index] = inputLine[i];
            index++;
            break;
        }
      }
    }
    lastName++;
  }

  if (stream) {
    err = fclose(stream);
    if (err != 0) {
      result = ERROR_CANNOT_CLOSE_FILE;
      goto error;
    }
  }

  err = _tfopen_s(&stream, _T("firstname.txt"), _T("r"));
  if (err != 0) {
    result = ERROR_CANNOT_OPEN_FILE;
    goto error;
  }
  
  while (NULL != _fgetts(inputLine, MAX_CHAR, stream)) firstNameCount++;
  rewind(stream);

  firstNameInitialValue = (name *) calloc(firstNameCount, sizeof(name));
  firstName = firstNameInitialValue;
  while (NULL != _fgetts(inputLine, MAX_CHAR, stream)) {
    int column = 0;
    int index = 0;
    for (unsigned int i = 0; i < _tcslen(inputLine); i++) {
      if (inputLine[i] == ',') {
        switch (column) {
          case 0:
            firstName->Kanji[index] = _T('\0');
            index = 0;
            break;
          case 1:
            firstName->Hiragana[index] = _T('\0');
            index = 0;
            break;
        }
        column++;
      } else {
        switch (column) {
          case 0:
            firstName->Kanji[index] = inputLine[i];
            index++;
            break;
          case 1:
            firstName->Hiragana[index] = inputLine[i];
            index++;
            break;
          case 2:
            switch (inputLine[i]) {
              case _T('0'):
                firstName->IsMale = false;
                break;
              case _T('1'):
                firstName->IsMale = true;
                break;
            }
            index++;
            break;
        }
      }
    }
    firstName++;
  }

  if (stream) {
    err = fclose(stream);
    if (err != 0) {
      result = ERROR_CANNOT_CLOSE_FILE;
      goto error;
    }
  }

  for (int i = 0; i < howMuchData; i++) {
    firstName = firstNameInitialValue + getRandomNumber(firstNameCount);
    lastName = lastNameInitialValue + getRandomNumber(lastNameCount);

    _tprintf(_T("%s%s%s%s%d\n"), 
      fixLength(lastName->Hiragana), fixLength(firstName->Hiragana),
      fixLength(lastName->Kanji), fixLength(firstName->Kanji), 
      firstName->IsMale);
  }

error:
  if (firstNameInitialValue) free(firstNameInitialValue);
  if (lastNameInitialValue) free(lastNameInitialValue);
  return result;
}

int getRandomNumber(int max) {
  if (max < 1) return -1;
  unsigned int number;
  errno_t err = rand_s(&number);
  if (err != 0) {
    _tprintf(_T("rand_s֐s\n"));
    return -1;
  } else {
    return (int) ((double) number / (double) UINT_MAX * 
      (double) max);
  }
}

TCHAR *fixLength(TCHAR *s) {
  TCHAR output[MAX_CHAR];
  int isEnd = false;
  for (unsigned int i = 0; i < NameColumnWidth; i++) {
    if (s[i] != _T('\0')) {
      if (isEnd) {
        output[i] = PadCharacter;
      } else {
        output[i] = s[i];
      }
    } else {
      output[i] = PadCharacter;
      isEnd = true;
    }
  }
  output[NameColumnWidth] = _T('\0');
  _tcscpy_s(s, MAX_CHAR, output);
  return s;
}
